{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
program lop_example(input, output);

use     i_ari,                           { Interval Arithmetic               }
        mv_ari,                          { Matrix/Vector Arithmetic          }
        mvi_ari,                         { Matrix/Vector Interval Arithmetic }
        lop,                             { Linear Optimization Module        }
        rev_simp;                        { Revised Simplex Algorithm Module  }

const   maxSolNo = 50;
var     m,n : integer;

procedure main(m,n : integer);
var
        A                       : rmatrix[1..m,1..n];
        c, x_start              : rvector[1..n];
        b, B_start              : rvector[1..m];
        V_opt                   : rmatrix[1..maxSolNo,1..m];
        X_opt                   : imatrix[1..maxSolNo,1..n];
        z_start                 : real;
        z                       : interval;
        Error, i, NoOfSolutions : integer;

begin {main}

  Error := 0;

  writeln;                                             { Read optimization   }
  writeln('Enter objective function (c[1]..c[n]): ');  { problem P = (A,b,c) }
  read(c); writeln;                                    {---------------------}
  writeln('Enter matrix of constraints (A[1,1]..A[m,n]): ');
  read(A); writeln;                             
  writeln('Enter right-hand side (b[1]..b[m]): ');
  read(b); writeln;
                                                         { Call revised      }
  RevSimplex(A, b, c, x_start, B_start, z_start, Error); { simplex procedure }
                                                         {-------------------}
  if (Error <> 0) then writeln('Error occurred during approximation')
  else 
    begin                                               { Display results of }
      writeln('Results of approximation:');             { calculation        }
      writeln('Optimal value :  ',z_start); writeln;    {--------------------}
      write('Initial basic index set : ');
      writeln(B_start);
      write('Initial optimal solution : ');
      writeln(x_start);
    end;

  if (Error = 0) then                                { Call verifying linear }
    begin                                            { optimization solver   }
      LinOpt(A, b, c, B_start, z, V_opt, X_opt, NoOfSolutions, Error);

      writeln('Results of verification:');
      if (Error = 0) or (Error = 6) or (Error = 7) then 
        begin                                              { Display results }
          writeln('Optimal value interval :  ',z); writeln;{ of calculation  }
          writeln('List of optimal basic index sets : ');  {-----------------}
          for i:=1 to NoOfSolutions do 
            begin
              write('B',i:1,' ='); write(V_opt[i]);
            end;
          writeln;
          writeln('List of optimal basic feasible solutions : ');
          for i:=1 to NoOfSolutions do 
            begin
              write('x of B',i:1,' ='); write(X_opt[i]);
            end;
          writeln;
        end;
    end;

  if (Error <> 0) then
    writeln(LinOptErrMsg(Error));

end; { procedure main }

begin

  writeln('Enter dimensions of linear ',              { Read dimension of    }
          'optimization problem (m and n): ');        { optimization problem }
  read(m); read(n);
  
  main(m,n);

end.
