{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
program linsys_example ( input, output );
use
  mv_ari,     { Matrix/vector arithmetic          }
  mvi_ari,    { Matrix/vector interval arithmetic }
  matinv,     { Matrix inversion     }
  linsys;     { Linear system solver }
{----------------------------------------------------------------------------}
procedure main ( n : integer );
var
  ErrCode : integer;
  Cond    : real;
  A, R    : rmatrix[1..n,1..n];
  b       : rvector[1..n];
  x       : ivector[1..n];
begin
  writeln('Enter matrix A:'); read(A); writeln;
  writeln('Enter vector b:'); read(b); writeln;

  LinSolve(A,b,x,Cond,ErrCode);

  if (ErrCode = 0) then
    begin
      { Compare the result to a naive floating-point approximation }
      MatInv(A,R,ErrCode);
      write('Naive floating-point approximation:');
      writeln(R*b);

      write('Verified solution found in:');
      writeln(x);
    end
  else
    writeln(LinSolveErrMsg(ErrCode));

  if (ErrCode in [0,4]) then
  begin
    writeln('Condition estimate:');
    writeln(Cond:9);
  end;

end;
{----------------------------------------------------------------------------}
var
  n : integer;
begin
  n := 0;
  while (n <= 0) do
    begin  write('Enter the dimension of the system: ');  read(n);  end;
  writeln;
  main(n);
end.
