{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
{----------------------------------------------------------------------------}
{ This program uses module 'nlfzero' to compute the zeros of the function    }
{                                                                            }
{              exp(-3*x) - power(sin(x), 3)                                  }
{                                                                            }
{ A starting interval and a tolerance must be entered.                       }
{----------------------------------------------------------------------------}
program nlfz_ex;
use
  i_ari,    { Interval arithmetic        }
  ddf_ari,  { Differentiation arithmetic }
  b_util,   { Boolean utilities          }
  nlfzero;  { Nonlinear function zeros   }
const
  n = 20;   { Maximum number of zeros to be computed }

function f (x: DerivType) : DerivType;
begin
  f:= exp(-3*x) - power(sin(x),3);
end;

{----------------------------------------------------------------------------}
{ Procedure for prompting and reading information to call the procedure      }
{ 'AllZeros'. This procedure must be called with the function 'f' and a      }
{ string 'Name' containing a textual description of that function.           }
{----------------------------------------------------------------------------}
procedure compute (function f(x:DerivType): DerivType; Name: string);
var
  SearchInterval            : interval;
  Tolerance                 : real;
  Zero                      : ivector[1..n];
  Unique                    : bvector[1..n];
  NumberOfZeros, i, ErrCode : integer;
begin
  writeln('Computing all zeros of the function  ', Name);
  write('Search interval      : ');  read(SearchInterval);
  write('Tolerance (relative) : ');  read(Tolerance);
  writeln;
  AllZeros(f,SearchInterval,Tolerance,Zero,Unique,NumberOfZeros,ErrCode);
  for i:=1 to NumberOfZeros do
  begin
    write(Zero[i]);
    if unique[i] then
      writeln(' encloses a locally unique zero!')
    else
      begin
        writeln(' may contain a zero');
        write(' ':52);  writeln(' (not verified unique)!')
      end;
  end;
  if ErrCode <> 0 then writeln(AllZerosErrMsg(ErrCode));
  writeln;  writeln(NumberOfZeros:1, ' interval enclosure(s)');  writeln;
end;

begin
  compute(f, 'EXP(-3x)-POWER(SIN(x),3)');
end.
