{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
program rpeval_example(input, output);
use
  i_ari,                       { Interval arithmetic                         }
  rpoly,                       { Real polynomials                            }
  rpeval;                      { Polynomial evaluation with maximum accuracy }

procedure main(n : integer);
var
  ErrCode, No : integer;
  t, y        : real;
  yy          : interval;
  p           : RPolynomial[0..n];

begin
  writeln('Enter the coefficients of p in increasing order:'); read(p);
  writeln;
  write('Enter the argument t for evaluation: '); read(t); writeln;

  No := 0;
  RPolyEval(p, t, y, yy, No, ErrCode);

  if (ErrCode = 0) then
    begin
      writeln('Polynomial: '); writeln(p);
      writeln('Floating-point evaluation of p(t) using Horner''s scheme:');
      writeln('  ',y); writeln;
      writeln('Verified inclusion of p(t): '); writeln(yy); writeln;
      writeln('Number of iterations needed: ', No);
    end
  else
    writeln(RPolyEvalErrMsg(ErrCode));
end; { procedure main }

var
  n : integer;

begin
  n := -1;
  while (n < 0) do
  begin
    write('Enter the degree of the polynomial (>=0): ');
    read(n); writeln;
  end;
  main(n);
end.
