{****************************************************************************}
{*                                                                          *}
{*                              Program/Module                              *}
{*                                   from                                   *}
{*                 NUMERICAL TOOLBOX FOR VERIFIED COMPUTING I               *}
{*                         Basic Numerical Problems                         *}
{*                                                                          *}
{*      Copyright (c) 1993   Rolf Hammer, Matthias Hocks, Dietmar Ratz      *}
{*                                                                          *}
{* This program/module is free software for non-commercial use. For details *}
{* on theory, algorithms, and programs, see the book                        *}
{*                                                                          *}
{*  R. Hammer, M. Hocks, U. Kulisch, D. Ratz:  Numerical Toolbox for        *}
{*  Verified Computing I - Basic Numerical Problems. Springer-Verlag,       *}
{*  Heidelberg (ISBN 3-540-57118-3), New York (ISBN 0-387-57118-3), 1993.   *}
{*                                                                          *}
{* This program/module is distributed WITHOUT ANY WARRANTY. For details,    *}
{* see the "Disclaimer / Legal Matters" of the book (page iv).              *}
{*                                                                          *}
{****************************************************************************}
{----------------------------------------------------------------------------}
{ Example for the evaluation of an arithmetic expression using procedure     }
{ 'Eval()'. Evaluate the following expression:                               }
{                                                                            }
{    f(x,y) := 1 / ( y^6 - 3*x*y^5 + 5*x^3*y^3 - 3*x^5*y - x^6 ).            }
{                                                                            }
{ With x and y being successive Fibonacci numbers defined by                 }
{                                                                            }
{    a_0 := 0,  a_1 := 1,  a_i := a_i-1 + a_i-2, i >= 2,                     }
{                                                                            }
{ we always get f(a_i,a_i+1) = (-1)^i. For instance a_66 = 27777890035288    }
{ and a_67 := 44945570212853 are such two successive Fibonacci numbers.      }
{----------------------------------------------------------------------------}
program ExprEval_Example_Fibonacci ( input, output );
use
  i_ari,        { Interval arithmetic   }
  expreval;     { Expression evaluation }

function f ( v : StaggArray ) : Staggered;
var
  x, y : Staggered;
begin
  x := v[1];
  y := v[2];
  f := 1 / ( Power(y,6) - 3*x*Power(y,5) + 5*Power(x,3)*Power(y,3)
                        - 3*Power(x,5)*y - Power(x,6) );
end;

var
  Eps, Approx        : real;
  StaggPrec, ErrCode : integer;
  Arg                : rvector[1..2];
  Encl               : interval;
begin
  writeln('Evaluation of 1 / ( y^6 - 3*x*y^5 + 5*x^3*y^3 - 3*x^5*y - x^6 )');
  writeln;
  writeln('Enter the arguments:');
  write  ('   x = '); read(Arg[1]);
  write  ('   y = '); read(Arg[2]);
  writeln;
  writeln('Desired accuracy:');
  write  ('   Eps = '); read(Eps);
  writeln;

  Eval(f,Arg,Eps,Approx,Encl,StaggPrec,ErrCode);
  if (ErrCode = 0) then
    begin
      writeln('Floating-point evaluation:  ', Approx);
      writeln('Interval enclosure:       ',   Encl);
      writeln('Corrections needed:          ',StaggPrec);
    end
  else
    writeln(EvalErrMsg(ErrCode));
end.
